
/*
 *                            COPYRIGHT
 *
 *  XAnalyser, frequence spectrum analyser for X Windows
 *  Copyright (C) 1998 Arvin Schnell
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Contact addresses:
 *  aschnell@physik.uni-bremen.de or arvinschnell@yahoo.com
 *  Arvin Schnell, Am Heidberg 8, 28865 Lilienthal, Germany
 *
 */


#include <stdlib.h>
#include <iostream.h>

#include <X11/Xlib.h>
#include <X11/xpm.h>
#include <Xm/Xm.h>
#include <Xm/MainW.h>
#include <Xm/AtomMgr.h>
#include <Xm/Protocols.h>

#include "main.hh"
#include "control.hh"
#include "display.hh"
#include "calc.hh"
#include "misc.hh"
#include "xanalyser.xpm"



char* program_name;

Widget shell, toplevel;
XtAppContext app_context;

Pixmap icon_pixmap, icon_mask;

Atom wm_delete_window;


// Xresources

XAnalyser xanalyser;

XtResource resources[] = {
  
  { "samplespeed", "SampleSpeed", XtRInt, sizeof( int ),
    XtOffsetOf( XAnalyser, sample_speed ), XtRInt, NULL },
  { "samplelength", "SampleLength", XtRInt, sizeof( int ),
    XtOffsetOf( XAnalyser, sample_length ), XtRInt, NULL },
  
  { "upperdb", "UpperDB", XtRFloat, sizeof( float ),
    XtOffsetOf( XAnalyser, upper_db ), XtRFloat, NULL },
  { "lowerdb", "LowerDB", XtRFloat, sizeof( float ),
    XtOffsetOf( XAnalyser, lower_db ), XtRFloat, NULL },
  
  { "alpha", "Alpha", XtRFloat, sizeof( float ),
    XtOffsetOf( XAnalyser, alpha ), XtRFloat, NULL },
  { "decay", "Decay", XtRFloat, sizeof( float ),
    XtOffsetOf( XAnalyser, decay ), XtRFloat, NULL },
  
  { "source", "Source", XtRString, sizeof( String ),
    XtOffsetOf( XAnalyser, source ), XtRString, NULL },
  { "filename", "Filename", XtRString, sizeof( String ),
    XtOffsetOf( XAnalyser, filename ), XtRString, NULL },
  { "channels", "Channels", XtRString, sizeof( String ),
    XtOffsetOf( XAnalyser, channels ), XtRString, NULL },
  
  { "datacolor", "DataColor", XtRPixel, sizeof( Pixel ),
    XtOffsetOf( XAnalyser, datacolor ), XtRString, NULL },
  { "backgroundcolor", "BackgroundColor", XtRPixel, sizeof( Pixel ),
    XtOffsetOf( XAnalyser, backgroundcolor ), XtRString, NULL },
  { "markercolor", "MarkerColor", XtRPixel, sizeof( Pixel ),
    XtOffsetOf( XAnalyser, markercolor ), XtRString, NULL },
  { "majorgridcolor", "MajorGridColor", XtRPixel, sizeof( Pixel ),
    XtOffsetOf( XAnalyser, majorgridcolor ), XtRString, NULL },
  { "minorgridcolor", "MinorGridColor", XtRPixel, sizeof( Pixel ),
    XtOffsetOf( XAnalyser, minorgridcolor ), XtRString, NULL },
  { "gridfont", "GridFont", XtRFontStruct, sizeof( XFontStruct* ),
    XtOffsetOf( XAnalyser, gridfont ), XtRString, NULL }
  
};


XrmOptionDescRec cmdline_options[] = {
  { "-speed",     ".samplespeed",   XrmoptionSepArg,  "" },
  { "-length",    ".samplelength",  XrmoptionSepArg,  "" },
  
  { "-soundcard", ".source",        XrmoptionNoArg,  "SOUNDCARD" },
  { "-diskfile",  ".source",        XrmoptionNoArg,  "DISKFILE" },
  { "-filename",  ".filename",      XrmoptionSepArg,  "" },
  
  { "-mono",      ".channels",      XrmoptionNoArg,  "MONO" },
  { "-left",      ".channels",      XrmoptionNoArg,  "LEFT" },
  { "-right",     ".channels",      XrmoptionNoArg,  "RIGHT" },
  { "-stereo",    ".channels",      XrmoptionNoArg,  "STEREO" }
};


String fallbacks[] = {
#include "XAnalyser.ad.h"
  NULL
};



void volatile
usage( void )
{
  cerr << "usage: " << program_name << " [-speed speed] [-length length] "
       << "[-soundcard] [-diskfile] [-file file] [-mono] [-left] [-right] "
       << "[-stereo]\n";
  exit( 1 );
}



int
main( int argc, char *argv[] )
{
  program_name = argv[0];
  
  shell = XtAppInitialize( &app_context, "XAnalyser",
			   cmdline_options, XtNumber( cmdline_options ),
			   &argc, argv, fallbacks, NULL, 0 );
  
  if( argc != 1 )
    usage();
  
  XtGetApplicationResources( shell, (XtPointer) &xanalyser,
                             resources, XtNumber( resources ),
                             NULL, 0 );
  
  
  toplevel = XmCreateMainWindow( shell, "MainWindow", NULL, 0 );
  XtManageChild( toplevel );
  
  
  control_create();
  
  
  XtRealizeWidget( shell );
  
  wm_delete_window = XmInternAtom( XtDisplay( toplevel ),
				   "WM_DELETE_WINDOW", False );
  
  XmAddWMProtocolCallback( shell, wm_delete_window, quit_cb, NULL );
  
  
  // icon
  
  Display *display;
  display = XtDisplay( shell );
  
  XpmCreatePixmapFromData( display, XtWindow( shell ),
                           icon_xpm, &icon_pixmap, &icon_mask, 0 );
  
  // FIXME, under high cpu/X11 load the icon does not appear
  // usleep( 500 );  // works (sometimes), but I don't like it
  
  XtVaSetValues( shell, XtNiconPixmap, icon_pixmap,
                 XtNiconMask, icon_mask, NULL );
  
  
  // you don't dare to change the order - FIXME, it's terrible
  
  control_set_default();
  calc_parms();
  calc_init();
  display_manage();
  display_remanage();
  set_sensitivity();
  
  
  XtAppMainLoop( app_context );
}

